@@ -10,8 +10,6 @@
 
 #include "AlphIO.h"
 
-#include <iostream>
-
 using namespace Dasher;
 using namespace std;
 using namespace expat;
@@ -26,11 +24,11 @@ static char THIS_FILE[] = __FILE__;
 #endif
 #endif
 
-CAlphIO::CAlphIO(string SystemLocation, string UserLocation, vector <string >Filenames)
+CAlphIO::CAlphIO(std::string SystemLocation, std::string UserLocation, std::vector<std::string> Filenames)
 :BlankInfo(), SystemLocation(SystemLocation), UserLocation(UserLocation), Filenames(Filenames), LoadMutable(false), CData("") {
   CreateDefault();
 
-  typedef pair < Opts::AlphabetTypes, string > AT;
+  typedef pair < Opts::AlphabetTypes, std::string > AT;
   vector < AT > Types;
   Types.push_back(AT(Opts::MyNone, "None"));
   Types.push_back(AT(Opts::Arabic, "Arabic"));
@@ -69,7 +67,6 @@ CAlphIO::CAlphIO(string SystemLocation, string UserLocation, vector <string >Fil
 }
 
 void CAlphIO::ParseFile(std::string Filename) {
-
   FILE *Input;
   if((Input = fopen(Filename.c_str(), "r")) == (FILE *) 0) {
     // could not open file
@@ -110,28 +107,24 @@ void CAlphIO::GetAlphabets(std::vector <std::string >*AlphabetList) const {
     AlphabetList->push_back((*Cur).second.AlphID);
 }
 
-const CAlphIO::AlphInfo & CAlphIO::GetInfo(const std::string &AlphID) {
-  if(AlphID == "") {
-    // FIXME - really broken, code duplicated in Win32/Widgets/AlphabetBox.cpp
-    // Eww. If no alphabet is configured, default to this one...
-    if(Alphabets.count("English with limited punctuation") != 0) {
-      return Alphabets["English with limited punctuation"];
-    }
-    else {
-      // unless it doesn't exist, in which case return default
-      return Alphabets["Default"];
-    }
+std::string CAlphIO::GetDefault() {
+  if(Alphabets.count("English with limited punctuation") != 0) {
+    return "English with limited punctuation";
   }
   else {
-    if(Alphabets.count(AlphID) != 0) {
-      // if we have the alphabet they ask for, return it
-      Alphabets[AlphID].AlphID = AlphID;        // Ensure consistency
-      return Alphabets[AlphID];
-    }
-    else {
-      // otherwise, give them default - it's better than nothing
-      return Alphabets["Default"];
-    }
+    return "Default";
+  }
+}
+
+const CAlphIO::AlphInfo & CAlphIO::GetInfo(const std::string &AlphID) {
+  if(Alphabets.count(AlphID) != 0) {
+    // if we have the alphabet they ask for, return it
+    Alphabets[AlphID].AlphID = AlphID;        // Ensure consistency
+    return Alphabets[AlphID];
+  }
+  else {
+    // otherwise, give them default - it's better than nothing
+    return Alphabets["Default"];
   }
 }
 
@@ -160,7 +153,7 @@ void CAlphIO::Save(const std::string &AlphID) {
   // overhead doesn't seem to matter and it makes things much easier.
   char Number[sizeof(int)];
   FILE *Output;
-  string Filename = UserLocation + "alphabet.xml";
+  std::string Filename = UserLocation + "alphabet.xml";
   if((Output = fopen(Filename.c_str(), "w")) == (FILE *) 0) {
     // could not open file
   }
@@ -321,7 +314,7 @@ void CAlphIO::CreateDefault() {
   Default.TrainingFile = "training_english_GB.txt";
   Default.GameModeFile = "gamemode_english_GB.txt";
   Default.PreferredColours = "Default";
-  string Chars = "abcdefghijklmnopqrstuvwxyz";
+  std::string Chars = "abcdefghijklmnopqrstuvwxyz";
 
 //   // Obsolete
 //   Default.Groups.resize(1);
@@ -348,7 +341,7 @@ void CAlphIO::CreateDefault() {
 void CAlphIO::XML_Escape(std::string *Text, bool Attribute) {
   // The XML "W3C Recommendation" is here: http://www.w3.org/TR/REC-xml
 
-  string & Input = *Text;       // Makes syntax less fiddly below
+  std::string & Input = *Text;       // Makes syntax less fiddly below
 
   for(unsigned int i = 0; i < Text->size(); i++) {
     // & and < need escaping in XML. In one rare circumstance >
@@ -383,7 +376,7 @@ void CAlphIO::XML_Escape(std::string *Text, bool Attribute) {
 // Below here handlers for the Expat XML input library
 ////////////////////////////////////////////////////////////////////////////////////
 
-void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_Char **atts) {
+void CAlphIO::XML_StartElement(void *userData, const expat::XML_Char *name, const expat::XML_Char **atts) {
   CAlphIO *Me = (CAlphIO *) userData;
 
   Me->CData = "";
@@ -395,9 +388,12 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     Me->InputInfo.SpaceCharacter.Colour = -1;
     Me->InputInfo.ParagraphCharacter.Colour = -1;
     Me->InputInfo.ControlCharacter.Colour = -1;
+    Me->InputInfo.StartConvertCharacter.Text = "";
+    Me->InputInfo.EndConvertCharacter.Text = "";
     Me->InputInfo.m_iCharacters = 1; // Start at 1 as 0 is the root node symbol
     Me->InputInfo.m_pBaseGroup = 0;
     Me->bFirstGroup = true;
+    Me->iGroupIdx = 0;
     while(*atts != 0) {
       if(strcmp(*atts, "name") == 0) {
         atts++;
@@ -406,6 +402,9 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
       }
       atts += 2;
     }
+    Me->InputInfo.m_iConversionID = 0;
+    Me->InputInfo.m_strDefaultContext = ". ";
+
     return;
   }
 
@@ -523,13 +522,10 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
   }
 
   if(strcmp(name, "group") == 0) {
-//     AlphInfo::group NewGroup;
-//     NewGroup.Colour = -1;
-//     NewGroup.Label = "";
-//     Me->InputInfo.Groups.push_back(NewGroup);
-    
     SGroupInfo *pNewGroup(new SGroupInfo);
-    pNewGroup->iColour = 0;
+    pNewGroup->iColour = (Me->iGroupIdx % 3) + 110; 
+    ++Me->iGroupIdx;
+
     if(Me->bFirstGroup) {
       pNewGroup->bVisible = false;
       Me->bFirstGroup = false;
@@ -540,31 +536,27 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
 
     while(*atts != 0) {
       if(strcmp(*atts, "name") == 0) {
+	// TODO: Fix this, or remove if names aren't needed
+
 //         atts++;
 //         Me->InputInfo.Groups.back().Description = *atts;
 //         atts--;
       }
       if(strcmp(*atts, "b") == 0) {
         atts++;
-	//        Me->InputInfo.Groups.back().Colour = atoi(*atts);
 	pNewGroup->iColour = atoi(*atts);
-// 	if(pNewGroup->iColour == 0)
-// 	  pNewGroup->bVisible = false;
-// 	else
-// 	  pNewGroup->bVisible = true;
         atts--;
       }
       if(strcmp(*atts, "visible") == 0) {
 	atts++;
-	if(!strcmp(*atts, "yes"))
+	if(!strcmp(*atts, "yes") || !strcmp(*atts, "on"))
 	  pNewGroup->bVisible = true;
-	else if(!strcmp(*atts, "no"))
+	else if(!strcmp(*atts, "no") || !strcmp(*atts, "off"))
 	  pNewGroup->bVisible = false;
 	atts--;
       }
       if(strcmp(*atts, "label") == 0) {
         atts++;
-	//        Me->InputInfo.Groups.back().Label = *atts;
 	pNewGroup->strLabel = *atts;
         atts--;
       }
@@ -590,7 +582,20 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     return;
   }
 
-  //-- Added for Kanji Conversion by T.Kaburagi 13 July 2005 START
+  if(!strcmp(name, "conversionmode")) {
+    while(*atts != 0) {
+      if(strcmp(*atts, "id") == 0) {
+        atts++;
+        Me->InputInfo.m_iConversionID = atoi(*atts);
+        atts--;
+      }
+      atts += 2;
+    }
+
+    return;
+  }
+
+  // Special characters for character composition
   if(strcmp(name, "convert") == 0) {
     while(*atts != 0) {
       if(strcmp(*atts, "t") == 0) {
@@ -644,7 +649,18 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     }
     return;
   }
-  //-- Added for Kanji Conversion by T.Kaburagi 13 July 2005 END
+
+  if(strcmp(name, "context") == 0) {
+    while(*atts != 0) {
+      if(strcmp(*atts, "default") == 0) {
+        atts++;
+        Me->InputInfo.m_strDefaultContext = *atts;
+        atts--;
+      }
+      atts += 2;
+    }
+    return;
+  }
 
   if(strcmp(name, "s") == 0) {
     AlphInfo::character NewCharacter;
@@ -688,7 +704,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
   }
 }
 
-void CAlphIO::XML_EndElement(void *userData, const XML_Char *name) {
+void CAlphIO::XML_EndElement(void *userData, const expat::XML_Char *name) {
   CAlphIO *Me = (CAlphIO *) userData;
 
   if(strcmp(name, "alphabet") == 0) {
@@ -712,14 +728,13 @@ void CAlphIO::XML_EndElement(void *userData, const XML_Char *name) {
   }
 
   if(!strcmp(name, "group")) {
-    //    Me->InputInfo.Groups.pop_back(NewGroup);
     Me->InputInfo.m_vGroups.back()->iEnd = Me->InputInfo.m_iCharacters;
     Me->InputInfo.m_vGroups.pop_back();
     return;
   }
 }
 
-void CAlphIO::XML_CharacterData(void *userData, const XML_Char *s, int len) {
+void CAlphIO::XML_CharacterData(void *userData, const expat::XML_Char *s, int len) {
   // CAREFUL: s points to a string which is NOT null-terminated.
 
   CAlphIO *Me = (CAlphIO *) userData;
