@@ -29,28 +29,28 @@ static bool g_iTimeoutID = 0;
 
 // CDasherControl class definitions
 CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl) {
+  m_pPangoCache = NULL;
+  m_pScreen = NULL;
+
   m_pDasherControl = pDasherControl;
   m_pVBox = GTK_WIDGET(pVBox);
 
   Realize();
-
  
-  m_pKeyboardHelper = new CKeyboardHelper(this);
-  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
-
-  // Don't create the screen until we've been realised.
-
-  m_pScreen = NULL;
-
+  //  m_pKeyboardHelper = new CKeyboardHelper(this);
+  //  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
 }
 
 void CDasherControl::CreateLocalFactories() {
-   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherMouseInput(m_pEventHandler, m_pSettingsStore)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherMouseInput(m_pEventHandler, m_pSettingsStore)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CSocketInput(m_pEventHandler, m_pSettingsStore)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasher1DMouseInput(m_pEventHandler, m_pSettingsStore)));
 
+#ifdef JOYSTICK
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInput(m_pEventHandler, m_pSettingsStore, this)));
+  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInputDiscrete(m_pEventHandler, m_pSettingsStore, this)));
+#endif
   
-
   // Create locally cached copies of the mouse input objects, as we
   // need to pass coordinates to them from the timer callback
   
@@ -198,10 +198,10 @@ CDasherControl::~CDasherControl() {
     m_pPangoCache = NULL;
   }
 
-  if(m_pKeyboardHelper) {
-    delete m_pKeyboardHelper;
-    m_pKeyboardHelper = 0;
-  }
+//   if(m_pKeyboardHelper) {
+//     delete m_pKeyboardHelper;
+//     m_pKeyboardHelper = 0;
+//   }
 }
 
 bool CDasherControl::FocusEvent(GtkWidget *pWidget, GdkEventFocus *pEvent) {
@@ -305,7 +305,7 @@ void CDasherControl::ExternalEventHandler(Dasher::CEvent *pEvent) {
   }
   else if(pEvent->m_iEventType == EV_EDIT_CONTEXT) {
     CEditContextEvent *pEditContextEvent(static_cast < CEditContextEvent * >(pEvent));
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_context_request", pEditContextEvent->m_iMaxLength);
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_context_request", pEditContextEvent->m_iOffset, pEditContextEvent->m_iLength);
   }
   else if(pEvent->m_iEventType == EV_START) {
     g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_start");
@@ -352,22 +352,25 @@ void CDasherControl::WriteTrainFile(const std::string &strNewText) {
   close(fd);
 }
 
+// TODO: Sort these methods out
 void CDasherControl::ExternalKeyDown(int iKeyVal) {
-  if(m_pKeyboardHelper) {
-    int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
+//   if(m_pKeyboardHelper) {
+//     int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
 
-    if(iButtonID != -1)
-      KeyDown(get_time(), iButtonID);
-  }
+//     if(iButtonID != -1)
+//       KeyDown(get_time(), iButtonID);
+//   }
+  KeyDown(get_time(), iKeyVal);
 }
 
 void CDasherControl::ExternalKeyUp(int iKeyVal) {
-  if(m_pKeyboardHelper) {
-    int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
+//   if(m_pKeyboardHelper) {
+//     int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
     
-    if(iButtonID != -1)
-      KeyUp(get_time(), iButtonID);
-  }
+//     if(iButtonID != -1)
+//       KeyUp(get_time(), iButtonID);
+//   }
+  KeyUp(get_time(), iKeyVal);
 }
 
 void CDasherControl::HandleParameterNotification(int iParameter) {
@@ -379,8 +382,9 @@ void CDasherControl::HandleParameterNotification(int iParameter) {
     }
     break;
   case BP_GLOBAL_KEYBOARD:
-    if(m_pKeyboardHelper)
-      m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
+    // TODO: reimplement
+//     if(m_pKeyboardHelper)
+//       m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
     break;
   }
 }
@@ -397,6 +401,10 @@ int CDasherControl::TimerEvent() {
   int iRootHeight;
 
   gdk_drawable_get_size(gdk_get_default_root_window(), &iRootWidth, &iRootHeight);
+
+  if(GetLongParameter(LP_YSCALE) < 10)
+    SetLongParameter(LP_YSCALE, 10);
+
   y = (y - iRootHeight / 2) * 4096 / GetLongParameter(LP_YSCALE);
 
   m_p1DMouseInput->SetCoordinates(x, y);
@@ -453,16 +461,18 @@ gboolean CDasherControl::ButtonPressEvent(GdkEventButton *event) {
 
   // Take the focus if we click on the canvas
 
-  GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
-  gboolean *returnType;
+//   GdkEventFocus *focusEvent = (GdkEventFocus *) g_malloc(sizeof(GdkEventFocus));
+//   gboolean *returnType;
 
-  focusEvent->type = GDK_FOCUS_CHANGE;
-  focusEvent->window = (GdkWindow *) m_pCanvas;
-  focusEvent->send_event = FALSE;
-  focusEvent->in = TRUE;
+//   focusEvent->type = GDK_FOCUS_CHANGE;
+//   focusEvent->window = (GdkWindow *) m_pCanvas;
+//   focusEvent->send_event = FALSE;
+//   focusEvent->in = TRUE;
 
-  gtk_widget_grab_focus(GTK_WIDGET(m_pCanvas));
-  g_signal_emit_by_name(GTK_OBJECT(m_pCanvas), "focus_in_event", GTK_WIDGET(m_pCanvas), focusEvent, NULL, &returnType);
+//   gtk_widget_grab_focus(GTK_WIDGET(m_pCanvas));
+//   g_signal_emit_by_name(GTK_OBJECT(m_pCanvas), "focus_in_event", GTK_WIDGET(m_pCanvas), focusEvent, NULL, &returnType);
+
+  // No - don't take the focus - give it to the text area instead
   
   if(event->type == GDK_BUTTON_PRESS)
     HandleClickDown(get_time(), (int)event->x, (int)event->y);
@@ -490,12 +500,12 @@ gint CDasherControl::KeyReleaseEvent(GdkEventKey *event) {
 //       SetLongParameter(LP_BOOSTFACTOR, 100);
   }
   else {
-    if(m_pKeyboardHelper) {
-      int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
+//     if(m_pKeyboardHelper) {
+//       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
-      if(iKeyVal != -1)
-	KeyUp(get_time(), iKeyVal);
-    }
+//       if(iKeyVal != -1)
+// 	KeyUp(get_time(), iKeyVal);
+//     }
   }
 
   return 0;
@@ -507,12 +517,12 @@ gint CDasherControl::KeyPressEvent(GdkEventKey *event) {
   //  else if((event->keyval == GDK_Control_L) || (event->keyval == GDK_Control_R))
   //   SetLongParameter(LP_BOOSTFACTOR, 25);
   // else {
-    if(m_pKeyboardHelper) {
-      int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
+//     if(m_pKeyboardHelper) {
+//       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
-      if(iKeyVal != -1)
-	KeyDown(get_time(), iKeyVal);
-    }
+//       if(iKeyVal != -1)
+// 	KeyDown(get_time(), iKeyVal);
+//     }
     //  }
   return 0;
 }
