@@ -11,6 +11,7 @@
 #include <libgnome/libgnome.h>
 
 #include "GtkDasherControl.h"
+#include "KeyboardHelper.h"
 #include "Preferences.h"
 #include "dasher_lock_dialogue.h"
 #ifdef WITH_MAEMO
@@ -21,6 +22,10 @@
 /* Static instance of singleton, USE SPARINGLY */
 static DasherMain *g_pDasherMain = NULL; 
 
+// TODO: The following global variable makes control mode editing work
+// - this needs to be sorted out properly.
+static gboolean g_bSend = true;
+
 struct _DasherMainPrivate {
   // The glade XML file - TODO: this shouldn't be kept after interface has been loaded
   GladeXML *pGladeXML;
@@ -30,6 +35,8 @@ struct _DasherMainPrivate {
   DasherPreferencesDialogue *pPreferencesDialogue;
   DasherEditor *pEditor;
 
+  CKeyboardHelper *pKeyboardHelper;
+
   // Various widgets which need to be cached:
   GtkWidget *pBufferView;
   GtkWidget *pDivider;
@@ -127,6 +134,7 @@ static void dasher_main_menu_command(DasherMain *pSelf, GtkWidget *pWidget);
 static void dasher_main_command_import(DasherMain *pSelf);
 static void dasher_main_command_quit(DasherMain *pSelf);
 static void dasher_main_command_preferences(DasherMain *pSelf);
+static void dasher_main_command_preferences_alphabet(DasherMain *pSelf);
 static void dasher_main_command_tutorial(DasherMain *pSelf);
 static void dasher_main_command_help(DasherMain *pSelf);
 static void dasher_main_command_about(DasherMain *pself);
@@ -146,6 +154,7 @@ static void dasher_main_setup_window(DasherMain *pSelf);
 static void dasher_main_populate_controls(DasherMain *pSelf);
 static void dasher_main_connect_control(DasherMain *pSelf);
 static gboolean dasher_main_command(DasherMain *pSelf, const gchar *szCommand);
+static gint dasher_main_lookup_key(DasherMain *pSelf, guint iKeyVal);
 
 /* TODO: Various functions which haven't yet been rationalised */
 gboolean grab_focus();
@@ -178,6 +187,8 @@ extern "C" gboolean test_focus_handler(GtkWidget *pWidget, GtkDirectionType iDir
 
 extern "C" void handle_context_request(GtkDasherControl * pDasherControl, gint iOffset, gint iLength, gpointer data);
 extern "C" void handle_control_event(GtkDasherControl *pDasherControl, gint iEvent, gpointer data);
+extern "C" void handle_start_event(GtkDasherControl *pDasherControl, gpointer data);
+extern "C" gint dasher_main_key_snooper(GtkWidget *pWidget, GdkEventKey *pEvent, gpointer pUserData);
 
 /* Boilerplate code */
 static void 
@@ -203,6 +214,10 @@ dasher_main_init(DasherMain *pDasherMain) {
 
   pPrivate->pAppSettings = NULL;
   pPrivate->pEditor = NULL;
+  pPrivate->pPreferencesDialogue = NULL;
+
+  pPrivate->pKeyboardHelper = new CKeyboardHelper(NULL);
+
   pPrivate->bWidgetsInitialised = false;
 }
 
@@ -214,9 +229,21 @@ dasher_main_finalize(GObject *pObject) {
   DasherMain *pDasherMain = DASHER_MAIN(pObject);
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pDasherMain);
 
+  dasher_main_save_state(pDasherMain);
+
   /* TODO: Does unref really do the right thing - check the whole ref counting situation */
   if(pPrivate->pEditor)
     g_object_unref(pPrivate->pEditor);
+
+  if(pPrivate->pPreferencesDialogue)
+    g_object_unref(pPrivate->pPreferencesDialogue);
+
+  if(pPrivate->pAppSettings)
+    g_object_unref(pPrivate->pAppSettings);
+
+  gtk_widget_destroy(pPrivate->pMainWindow);
+
+  /* TDO: Do we need to take down anything else? */
 }
 
 /* Public methods */
@@ -298,6 +325,8 @@ dasher_main_new(int *argc, char ***argv, SCommandLine *pCommandLine) {
     dasher_main_populate_controls(pDasherMain);
     dasher_main_connect_control(pDasherMain);
 
+    gtk_key_snooper_install(dasher_main_key_snooper, pDasherMain);
+
     /* Cache a file-wide static pointer to the singleton class */
     g_pDasherMain = pDasherMain;
 
@@ -310,7 +339,7 @@ static void
 dasher_main_load_interface(DasherMain *pSelf) {
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
   
-  const char *szGladeFilename;
+  const char *szGladeFilename = NULL;
 
 #ifdef WITH_GPE
   szGladeFilename = PROGDATA "/dashergpe.glade";
@@ -341,6 +370,10 @@ dasher_main_load_interface(DasherMain *pSelf) {
   }
 #endif
 
+  if(!szGladeFilename) {
+    g_error("Failure to determine glade filename");
+  }
+
   pPrivate->pGladeXML = glade_xml_new(szGladeFilename, NULL, NULL);
 
   if (!pPrivate->pGladeXML) {
@@ -650,9 +683,8 @@ static void
 dasher_main_connect_control(DasherMain *pSelf) {
   /* TODO: This is very much temporary - we need to think of a better
      way of presenting application commands in a unified way */
-  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
-
 #ifdef GNOME_SPEECH
+  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
 
   gtk_dasher_control_register_node( GTK_DASHER_CONTROL(pPrivate->pDasherWidget), 
 				    Dasher::CControlManager::CTL_USER,
@@ -722,6 +754,11 @@ dasher_main_command(DasherMain *pSelf, const gchar *szCommand) {
     return TRUE;
   }
 
+  if(!strcmp(szCommand, "preferences_alphabet")) {
+    dasher_main_command_preferences_alphabet(pSelf);
+    return TRUE;
+  }
+
   if(!strcmp(szCommand, "tutorial")) {
     dasher_main_command_tutorial(pSelf);
     return TRUE;
@@ -856,8 +893,6 @@ dasher_main_connect_menus(DasherMain *pSelf) {
 
 static void 
 dasher_main_menu_command(DasherMain *pSelf, GtkWidget *pWidget) {
-  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
-
   int iNumItems = sizeof(MenuCommands) / sizeof(DasherMenuCommand);
 
   for(int i(0); i < iNumItems; ++i) {
@@ -904,9 +939,6 @@ static void dasher_main_command_quit(DasherMain *pSelf) {
 
   GtkWidget *pDialogue = NULL;
 
-  // TODO: reimplement if necessary
-  //  dasher_main_save_state(g_pSelf);
-
   if(dasher_editor_file_changed(pPrivate->pEditor)) {
     const gchar *szFilename = dasher_editor_get_filename(pPrivate->pEditor);
 
@@ -957,7 +989,13 @@ static void dasher_main_command_quit(DasherMain *pSelf) {
 static void 
 dasher_main_command_preferences(DasherMain *pSelf) { 
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
-  dasher_preferences_dialogue_show(pPrivate->pPreferencesDialogue);
+  dasher_preferences_dialogue_show(pPrivate->pPreferencesDialogue, 0);
+}
+
+static void 
+dasher_main_command_preferences_alphabet(DasherMain *pSelf) { 
+  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
+  dasher_preferences_dialogue_show(pPrivate->pPreferencesDialogue, 1);
 }
 
 static void 
@@ -1097,7 +1135,7 @@ dasher_main_alphabet_combo_changed(DasherMain *pSelf) {
   if(!strcmp("More Alphabets...", szSelected)) {
     gtk_combo_box_set_active(GTK_COMBO_BOX(pPrivate->pAlphabetCombo), 0);
     //    dasher_preferences_dialogue_show(pPrivate->pPreferencesDialogue);
-    dasher_main_command(pSelf, "preferences");
+    dasher_main_command(pSelf, "preferences_alphabet");
   }
   else 
     dasher_app_settings_set_string(pPrivate->pAppSettings, SP_ALPHABET_ID, szSelected);
@@ -1159,6 +1197,15 @@ dasher_main_populate_alphabet_combo(DasherMain *pSelf) {
 #endif
 }
 
+static gint
+dasher_main_lookup_key(DasherMain *pSelf, guint iKeyVal) {
+  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
+
+  if(pPrivate->pKeyboardHelper)
+    return pPrivate->pKeyboardHelper->ConvertKeycode(iKeyVal);
+  else
+    return -1;
+}
 
 
 gboolean 
@@ -1195,6 +1242,9 @@ extern "C" gboolean
 speed_changed(GtkWidget *pWidget, gpointer user_data) {
   if(g_pDasherMain)
     return dasher_main_speed_changed(g_pDasherMain);
+
+  // TODO: Check callback return functions
+  return false;
 }
 
 extern "C" void 
@@ -1218,6 +1268,9 @@ dasher_main_cb_buffer_changed(DasherEditor *pEditor, gpointer pUserData) {
 
 extern "C" void 
 dasher_main_cb_context_changed(DasherEditor *pEditor, gpointer pUserData) {
+  if(!g_bSend)
+    return;
+
   DasherMain *pDasherMain = DASHER_MAIN(pUserData);
   DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pDasherMain);
 
@@ -1267,6 +1320,9 @@ edit_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
 //   else {
 //     return false;
 //   }
+
+  // TODO: Check callback return functions
+  return false;
 }
 
 extern "C" gboolean 
@@ -1281,6 +1337,9 @@ edit_key_release(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
 //   else {
 //     return false;
 //   }
+
+  // TODO: Check callback return functions
+  return false;
 }
 
 extern "C" gboolean 
@@ -1311,14 +1370,69 @@ handle_control_event(GtkDasherControl *pDasherControl, gint iEvent, gpointer dat
   switch(iEvent) {
   case Dasher::CControlManager::CTL_USER + 1:
     dasher_main_command(g_pDasherMain, "speakall");
-    break;
+    return;
   case Dasher::CControlManager::CTL_USER + 2:
     dasher_main_command(g_pDasherMain, "speaklast");
-    break;
+    return;
   case Dasher::CControlManager::CTL_USER + 3:
     dasher_main_command(g_pDasherMain, "speakrepeat");
-    break;
+    return;
   default:
     break;
   }
+
+
+  // TODO: This is a horrible hack here to make the release work!  
+
+  g_bSend = false;
+
+  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+  dasher_editor_handle_control(pPrivate->pEditor, iEvent);
+
+  gtk_dasher_control_set_control_offset(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), 
+					dasher_editor_get_offset(pPrivate->pEditor));
+
+  g_bSend = true;
+  // ---
+}
+
+
+extern "C" void 
+handle_start_event(GtkDasherControl *pDasherControl, gpointer data) { 
+  if(!g_pDasherMain)
+    return;
+
+  DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(g_pDasherMain);
+  
+  dasher_editor_grab_focus(pPrivate->pEditor);
+}
+
+
+// TODO: Make this only work for children of the main window
+extern "C" gint 
+dasher_main_key_snooper(GtkWidget *pWidget, GdkEventKey *pEvent, gpointer pUserData) {
+  DasherMain *pSelf = DASHER_MAIN(pUserData);
+
+  gint iButton = dasher_main_lookup_key(pSelf, pEvent->keyval);
+  
+  if(iButton != -1) {
+    DasherMainPrivate *pPrivate = DASHER_MAIN_GET_PRIVATE(pSelf);
+
+    if(gdk_window_get_toplevel(pEvent->window) == pPrivate->pMainWindow->window) {
+      if(pPrivate->pDasherWidget) {
+	if(pEvent->type == GDK_KEY_PRESS)
+	  gtk_dasher_control_external_key_down(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), iButton);
+	else
+	  gtk_dasher_control_external_key_up(GTK_DASHER_CONTROL(pPrivate->pDasherWidget), iButton);
+      }
+      
+      return TRUE;
+    }
+    else {
+      return FALSE;
+    }
+  }
+  else {
+    return FALSE;
+  }
 }
